@@ -48,41 +48,39 @@ static const struct got_error *got_get_repo_commit(struct request *,
     struct repo_commit *, struct got_commit_object *, struct got_reflist_head *,
     struct got_object_id *);
 static const struct got_error *got_gotweb_dupfd(int *, int *);
-static const struct got_error *got_gotweb_openfile(FILE **, int *, int *);
+static const struct got_error *got_gotweb_openfile(FILE **, int *);
 static const struct got_error *got_gotweb_blame_cb(void *, int, int,
     struct got_commit_object *,struct got_object_id *);
 
 const struct got_error *
-got_gotweb_flushfile(FILE *f, int fd)
+got_gotweb_closefile(FILE *f)
 {
+	const struct got_error *err = NULL;
+
 	if (fseek(f, 0, SEEK_SET) == -1)
-		return got_error_from_errno("fseek");
+		err = got_error_from_errno("fseek");
 
-	if (ftruncate(fd, 0) == -1)
-		return got_error_from_errno("ftruncate");
+	if (err == NULL && ftruncate(fileno(f), 0) == -1)
+		err = got_error_from_errno("ftruncate");
 
-	if (fsync(fd) == -1)
-		return got_error_from_errno("fsync");
+	if (fclose(f) == EOF && err == NULL)
+		err = got_error_from_errno("fclose");
 
-	if (f && fclose(f) == EOF)
-		return got_error_from_errno("fclose");
-
-	if (fd != -1 && close(fd) != -1)
-		return got_error_from_errno("close");
-
-	return NULL;
+	return err;
 }
 
 static const struct got_error *
-got_gotweb_openfile(FILE **f, int *priv_fd, int *fd)
+got_gotweb_openfile(FILE **f, int *priv_fd)
 {
-	*fd = dup(*priv_fd);
-	if (*fd == -1)
+	int fd;
+
+	fd = dup(*priv_fd);
+	if (fd == -1)
 		return got_error_from_errno("dup");
 
-	*f = fdopen(*fd, "w+");
+	*f = fdopen(fd, "w+");
 	if (*f == NULL) {
-		close(*fd);
+		close(fd);
 		return got_error(GOT_ERR_PRIVSEP_NO_FD);
 	}
 
@@ -179,7 +177,7 @@ got_get_repo_age(time_t *repo_age, struct request *c, const char *refname)
 
 	if (cmp_time != 0)
 		*repo_age = cmp_time;
-done:
+ done:
 	got_ref_list_free(&refs);
 	return error;
 }
@@ -443,7 +441,7 @@ got_get_repo_commits(struct request *c, int limit)
 			commit = NULL;
 		}
 	}
-done:
+ done:
 	if (ref)
 		got_ref_close(ref);
 	if (commit)
@@ -669,7 +667,7 @@ got_get_repo_tags(struct request *c, int limit)
 		}
 	}
 
-done:
+ done:
 	/*
 	 * we have tailq populated, so find previous commit id
 	 * for navigation through briefs and commits
@@ -699,7 +697,7 @@ done:
 			c_cnt++;
 		}
 	}
-err:
+ err:
 	if (commit)
 		got_object_commit_close(commit);
 	if (tag)
@@ -773,7 +771,7 @@ got_output_repo_tree(struct request *c,
 			break;
 		}
 	}
-done:
+ done:
 	free(escaped_name);
 	free(path);
 	got_ref_list_free(&refs);
@@ -784,7 +782,8 @@ done:
 	free(commit_id);
 	free(tree_id);
 	if (error) {
-		log_warnx("%s: %s", __func__, error->msg);
+		if (error->code != GOT_ERR_CANCELLED)
+			log_warnx("%s: %s", __func__, error->msg);
 		return -1;
 	}
 	return 0;
@@ -903,8 +902,9 @@ got_output_blob_by_lines(struct template *tp, struct got_blob_object *blob,
 	free(line);
 
 	if (err) {
-		log_warnx("%s: got_object_blob_getline failed: %s",
-		    __func__, err->msg);
+		if (err->code != GOT_ERR_CANCELLED)
+			log_warnx("%s: got_object_blob_getline failed: %s",
+			    __func__, err->msg);
 		return -1;
 	}
 	return 0;
@@ -994,7 +994,7 @@ got_gotweb_blame_cb(void *arg, int nlines, int lineno,
 		a->lineno_cur++;
 		bline = &a->lines[a->lineno_cur - 1];
 	}
-done:
+ done:
 	free(line);
 	return err;
 }
@@ -1012,8 +1012,7 @@ got_output_file_blame(struct request *c, got_render_blame_line_cb cb)
 	struct got_blob_object *blob = NULL;
 	char *path = NULL, *in_repo_path = NULL;
 	struct blame_cb_args bca;
-	int i, obj_type, fd1 = -1, fd2 = -1, fd3 = -1, fd4 = -1, fd5 = -1;
-	int fd6 = -1;
+	int i, obj_type, blobfd = -1, fd1 = -1, fd2 = -1;
 	off_t filesize;
 	FILE *f1 = NULL, *f2 = NULL;
 
@@ -1054,15 +1053,15 @@ got_output_file_blame(struct request *c, got_render_blame_line_cb cb)
 		goto done;
 	}
 
-	error = got_gotweb_openfile(&bca.f, &c->priv_fd[BLAME_FD_1], &fd1);
+	error = got_gotweb_openfile(&bca.f, &c->priv_fd[BLAME_FD_1]);
 	if (error)
 		goto done;
 
-	error = got_gotweb_dupfd(&c->priv_fd[BLAME_FD_2], &fd2);
+	error = got_gotweb_dupfd(&c->priv_fd[BLAME_FD_2], &blobfd);
 	if (error)
 		goto done;
 
-	error = got_object_open_as_blob(&blob, repo, obj_id, BUF, fd2);
+	error = got_object_open_as_blob(&blob, repo, obj_id, BUF, blobfd);
 	if (error)
 		goto done;
 
@@ -1090,27 +1089,27 @@ got_output_file_blame(struct request *c, got_render_blame_line_cb cb)
 	bca.repo = repo;
 	bca.c = c;
 
-	error = got_gotweb_dupfd(&c->priv_fd[BLAME_FD_3], &fd3);
+	error = got_gotweb_dupfd(&c->priv_fd[BLAME_FD_3], &fd1);
 	if (error)
 		goto done;
 
-	error = got_gotweb_dupfd(&c->priv_fd[BLAME_FD_4], &fd4);
+	error = got_gotweb_dupfd(&c->priv_fd[BLAME_FD_4], &fd2);
 	if (error)
 		goto done;
 
-	error = got_gotweb_openfile(&f1, &c->priv_fd[BLAME_FD_5], &fd5);
+	error = got_gotweb_openfile(&f1, &c->priv_fd[BLAME_FD_5]);
 	if (error)
 		goto done;
 
-	error = got_gotweb_openfile(&f2, &c->priv_fd[BLAME_FD_6], &fd6);
+	error = got_gotweb_openfile(&f2, &c->priv_fd[BLAME_FD_6]);
 	if (error)
 		goto done;
 
 	error = got_blame(in_repo_path, commit_id, repo,
 	    GOT_DIFF_ALGORITHM_MYERS, got_gotweb_blame_cb, &bca, NULL, NULL,
-	    fd3, fd4, f1, f2);
+	    fd1, fd2, f1, f2);
 
-done:
+ done:
 	if (bca.lines) {
 		free(bca.line_offsets);
 		for (i = 0; i < bca.nlines; i++) {
@@ -1120,27 +1119,24 @@ done:
 		}
 	}
 	free(bca.lines);
+	if (blobfd != -1 && close(blobfd) == -1 && error == NULL)
+		error = got_error_from_errno("close");
+	if (fd1 != -1 && close(fd1) == -1 && error == NULL)
+		error = got_error_from_errno("close");
 	if (fd2 != -1 && close(fd2) == -1 && error == NULL)
 		error = got_error_from_errno("close");
-	if (fd3 != -1 && close(fd3) == -1 && error == NULL)
-		error = got_error_from_errno("close");
-	if (fd4 != -1 && close(fd4) == -1 && error == NULL)
-		error = got_error_from_errno("close");
 	if (bca.f) {
-		const struct got_error *bca_err =
-		    got_gotweb_flushfile(bca.f, fd1);
+		const struct got_error *bca_err = got_gotweb_closefile(bca.f);
 		if (error == NULL)
 			error = bca_err;
 	}
 	if (f1) {
-		const struct got_error *f1_err =
-		    got_gotweb_flushfile(f1, fd5);
+		const struct got_error *f1_err = got_gotweb_closefile(f1);
 		if (error == NULL)
 			error = f1_err;
 	}
 	if (f2) {
-		const struct got_error *f2_err =
-		    got_gotweb_flushfile(f2, fd6);
+		const struct got_error *f2_err = got_gotweb_closefile(f2);
 		if (error == NULL)
 			error = f2_err;
 	}
@@ -1157,7 +1153,7 @@ done:
 }
 
 const struct got_error *
-got_open_diff_for_output(FILE **fp, int *fd, struct request *c)
+got_open_diff_for_output(FILE **fp, struct request *c)
 {
 	const struct got_error *error = NULL;
 	struct transport *t = c->t;
@@ -1166,22 +1162,21 @@ got_open_diff_for_output(FILE **fp, int *fd, struct request *c)
 	struct got_object_id *id1 = NULL, *id2 = NULL;
 	struct got_reflist_head refs;
 	FILE *f1 = NULL, *f2 = NULL, *f3 = NULL;
-	int obj_type, fd1, fd2, fd3, fd4 = -1, fd5 = -1;
+	int obj_type, fd1 = -1, fd2 = -1;
 
 	*fp = NULL;
-	*fd = -1;
 
 	TAILQ_INIT(&refs);
 
-	error = got_gotweb_openfile(&f1, &c->priv_fd[DIFF_FD_1], &fd1);
+	error = got_gotweb_openfile(&f1, &c->priv_fd[DIFF_FD_1]);
 	if (error)
 		return error;
 
-	error = got_gotweb_openfile(&f2, &c->priv_fd[DIFF_FD_2], &fd2);
+	error = got_gotweb_openfile(&f2, &c->priv_fd[DIFF_FD_2]);
 	if (error)
 		return error;
 
-	error = got_gotweb_openfile(&f3, &c->priv_fd[DIFF_FD_3], &fd3);
+	error = got_gotweb_openfile(&f3, &c->priv_fd[DIFF_FD_3]);
 	if (error)
 		return error;
 
@@ -1205,28 +1200,28 @@ got_open_diff_for_output(FILE **fp, int *fd, struct request *c)
 	if (error)
 		goto done;
 
-	error = got_gotweb_dupfd(&c->priv_fd[DIFF_FD_4], &fd4);
+	error = got_gotweb_dupfd(&c->priv_fd[DIFF_FD_4], &fd1);
 	if (error)
 		goto done;
 
-	error = got_gotweb_dupfd(&c->priv_fd[DIFF_FD_5], &fd5);
+	error = got_gotweb_dupfd(&c->priv_fd[DIFF_FD_5], &fd2);
 	if (error)
 		goto done;
 
 	switch (obj_type) {
 	case GOT_OBJ_TYPE_BLOB:
-		error = got_diff_objects_as_blobs(NULL, NULL, f1, f2, fd4, fd5,
+		error = got_diff_objects_as_blobs(NULL, NULL, f1, f2, fd1, fd2,
 		     id1, id2, NULL, NULL, GOT_DIFF_ALGORITHM_MYERS, 3, 0, 0,
 		     NULL, repo, f3);
 		break;
 	case GOT_OBJ_TYPE_TREE:
-		error = got_diff_objects_as_trees(NULL, NULL, f1, f2, fd4, fd5,
+		error = got_diff_objects_as_trees(NULL, NULL, f1, f2, fd1, fd2,
 		    id1, id2, NULL, "", "",  GOT_DIFF_ALGORITHM_MYERS, 3, 0, 0,
 		    NULL, repo, f3);
 		break;
 	case GOT_OBJ_TYPE_COMMIT:
-		error = got_diff_objects_as_commits(NULL, NULL, f1, f2, fd4,
-		    fd5, id1, id2, NULL,  GOT_DIFF_ALGORITHM_MYERS, 3, 0, 0,
+		error = got_diff_objects_as_commits(NULL, NULL, f1, f2, fd1,
+		    fd2, id1, id2, NULL,  GOT_DIFF_ALGORITHM_MYERS, 3, 0, 0,
 		    NULL, repo, f3);
 		break;
 	default:
@@ -1235,45 +1230,31 @@ got_open_diff_for_output(FILE **fp, int *fd, struct request *c)
 	if (error)
 		goto done;
 
-	if (fseek(f1, 0, SEEK_SET) == -1) {
-		error = got_ferror(f1, GOT_ERR_IO);
-		goto done;
-	}
-
-	if (fseek(f2, 0, SEEK_SET) == -1) {
-		error = got_ferror(f2, GOT_ERR_IO);
-		goto done;
-	}
-
 	if (fseek(f3, 0, SEEK_SET) == -1) {
 		error = got_ferror(f3, GOT_ERR_IO);
 		goto done;
 	}
 
 	*fp = f3;
-	*fd = fd3;
 
-done:
-	if (fd4 != -1 && close(fd4) == -1 && error == NULL)
+ done:
+	if (fd1 != -1 && close(fd1) == -1 && error == NULL)
 		error = got_error_from_errno("close");
-	if (fd5 != -1 && close(fd5) == -1 && error == NULL)
+	if (fd2 != -1 && close(fd2) == -1 && error == NULL)
 		error = got_error_from_errno("close");
 	if (f1) {
-		const struct got_error *f1_err =
-		    got_gotweb_flushfile(f1, fd1);
+		const struct got_error *f1_err = got_gotweb_closefile(f1);
 		if (error == NULL)
 			error = f1_err;
 	}
 	if (f2) {
-		const struct got_error *f2_err =
-		    got_gotweb_flushfile(f2, fd2);
+		const struct got_error *f2_err = got_gotweb_closefile(f2);
 		if (error == NULL)
 			error = f2_err;
 	}
 	if (error && f3) {
-		got_gotweb_flushfile(f3, fd3);
+		got_gotweb_closefile(f3);
 		*fp = NULL;
-		*fd = -1;
 	}
 	got_ref_list_free(&refs);
 	free(id1);
@@ -1286,7 +1267,7 @@ got_init_repo_commit(struct repo_commit **rc)
 {
 	*rc = calloc(1, sizeof(**rc));
 	if (*rc == NULL)
-		return got_error_from_errno2("%s: calloc", __func__);
+		return got_error_from_errno2(__func__, "calloc");
 
 	(*rc)->path = NULL;
 	(*rc)->refs_str = NULL;
@@ -1305,7 +1286,7 @@ got_init_repo_tag(struct repo_tag **rt)
 {
 	*rt = calloc(1, sizeof(**rt));
 	if (*rt == NULL)
-		return got_error_from_errno2("%s: calloc", __func__);
+		return got_error_from_errno2(__func__, "calloc");
 
 	(*rt)->commit_id = NULL;
 	(*rt)->tag_name = NULL;
