@@ -287,7 +287,7 @@ gotweb_process_request(struct request *c)
 			log_warnx("%s: %s", __func__, error->msg);
 			goto err;
 		}
-		error = got_open_diff_for_output(&c->t->fp, &c->t->fd, c);
+		error = got_open_diff_for_output(&c->t->fp, c);
 		if (error) {
 			log_warnx("%s: %s", __func__, error->msg);
 			goto err;
@@ -417,20 +417,12 @@ gotweb_init_transport(struct transport **t)
 
 	*t = calloc(1, sizeof(**t));
 	if (*t == NULL)
-		return got_error_from_errno2("%s: calloc", __func__);
+		return got_error_from_errno2(__func__, "calloc");
 
 	TAILQ_INIT(&(*t)->repo_commits);
 	TAILQ_INIT(&(*t)->repo_tags);
 	TAILQ_INIT(&(*t)->refs);
 
-	(*t)->repo = NULL;
-	(*t)->repo_dir = NULL;
-	(*t)->qs = NULL;
-	(*t)->next_id = NULL;
-	(*t)->prev_id = NULL;
-	(*t)->next_disp = 0;
-	(*t)->prev_disp = 0;
-
 	(*t)->fd = -1;
 
 	return error;
@@ -443,13 +435,13 @@ gotweb_init_querystring(struct querystring **qs)
 
 	*qs = calloc(1, sizeof(**qs));
 	if (*qs == NULL)
-		return got_error_from_errno2("%s: calloc", __func__);
+		return got_error_from_errno2(__func__, "calloc");
 
 	(*qs)->headref = strdup("HEAD");
 	if ((*qs)->headref == NULL) {
 		free(*qs);
 		*qs = NULL;
-		return got_error_from_errno2("%s: strdup", __func__);
+		return got_error_from_errno2(__func__, "strdup");
 	}
 
 	(*qs)->action = INDEX;
@@ -474,7 +466,7 @@ gotweb_parse_querystring(struct querystring **qs, char *qst)
 
 	tok1 = strdup(qst);
 	if (tok1 == NULL)
-		return got_error_from_errno2("%s: strdup", __func__);
+		return got_error_from_errno2(__func__, "strdup");
 
 	tok1_pair = tok1;
 	tok1_end = tok1;
@@ -485,7 +477,7 @@ gotweb_parse_querystring(struct querystring **qs, char *qst)
 		tok2 = strdup(tok1_pair);
 		if (tok2 == NULL) {
 			free(tok1);
-			return got_error_from_errno2("%s: strdup", __func__);
+			return got_error_from_errno2(__func__, "strdup");
 		}
 
 		tok2_pair = tok2;
@@ -590,24 +582,24 @@ qa_found:
 		case COMMIT:
 			(*qs)->commit = strdup(value);
 			if ((*qs)->commit == NULL) {
-				error = got_error_from_errno2("%s: strdup",
-				    __func__);
+				error = got_error_from_errno2(__func__,
+				    "strdup");
 				goto done;
 			}
 			break;
 		case RFILE:
 			(*qs)->file = strdup(value);
 			if ((*qs)->file == NULL) {
-				error = got_error_from_errno2("%s: strdup",
-				    __func__);
+				error = got_error_from_errno2(__func__,
+				    "strdup");
 				goto done;
 			}
 			break;
 		case FOLDER:
 			(*qs)->folder = strdup(value);
 			if ((*qs)->folder == NULL) {
-				error = got_error_from_errno2("%s: strdup",
-				    __func__);
+				error = got_error_from_errno2(__func__,
+				    "strdup");
 				goto done;
 			}
 			break;
@@ -615,8 +607,8 @@ qa_found:
 			free((*qs)->headref);
 			(*qs)->headref = strdup(value);
 			if ((*qs)->headref == NULL) {
-				error = got_error_from_errno2("%s: strdup",
-				    __func__);
+				error = got_error_from_errno2(__func__,
+				    "strdup");
 				goto done;
 			}
 			break;
@@ -626,8 +618,8 @@ qa_found:
 			(*qs)->index_page = strtonum(value, INT64_MIN,
 			    INT64_MAX, &errstr);
 			if (errstr) {
-				error = got_error_from_errno3("%s: strtonum %s",
-				    __func__, errstr);
+				error = got_error_from_errno3(__func__,
+				    "strtonum", errstr);
 				goto done;
 			}
 			if ((*qs)->index_page < 0)
@@ -636,8 +628,8 @@ qa_found:
 		case PATH:
 			(*qs)->path = strdup(value);
 			if ((*qs)->path == NULL) {
-				error = got_error_from_errno2("%s: strdup",
-				    __func__);
+				error = got_error_from_errno2(__func__,
+				    "strdup");
 				goto done;
 			}
 			break;
@@ -647,8 +639,8 @@ qa_found:
 			(*qs)->page = strtonum(value, INT64_MIN,
 			    INT64_MAX, &errstr);
 			if (errstr) {
-				error = got_error_from_errno3("%s: strtonum %s",
-				    __func__, errstr);
+				error = got_error_from_errno3(__func__,
+				    "strtonum", errstr);
 				goto done;
 			}
 			if ((*qs)->page < 0)
@@ -742,14 +734,13 @@ gotweb_free_transport(struct transport *t)
 	if (t->blob)
 		got_object_blob_close(t->blob);
 	if (t->fp) {
-		err = got_gotweb_flushfile(t->fp, t->fd);
+		err = got_gotweb_closefile(t->fp);
 		if (err)
-			log_warnx("%s: got_gotweb_flushfile failure: %s",
+			log_warnx("%s: got_gotweb_closefile failure: %s",
 			    __func__, err->msg);
-		t->fd = -1;
 	}
-	if (t->fd != -1)
-		close(t->fd);
+	if (t->fd != -1 && close(t->fd) == -1)
+		log_warn("%s: close", __func__);
 	if (t->repos) {
 		for (i = 0; i < t->nrepos; ++i)
 			free(t->repos[i]);
