@@ -96,9 +96,11 @@
 #include <syslog.h>
 #include <unistd.h>
 
+#include "got_sockaddr.h"
+#include "got_reference.h"
+
 #include "proc.h"
 #include "gotwebd.h"
-#include "got_sockaddr.h"
 
 TAILQ_HEAD(files, file)		 files = TAILQ_HEAD_INITIALIZER(files);
 static struct file {
@@ -146,14 +148,10 @@ int		 get_addrs(const char *, struct server *, in_port_t);
 int		 addr_dup_check(struct addresslist *, struct address *,
 		    const char *, const char *);
 int		 add_addr(struct server *, struct address *);
-struct address	*host_v4(const char *);
-struct address	*host_v6(const char *);
-int		 host_dns(const char *, struct server *,
+int		 host(const char *, struct server *,
 		    int, in_port_t, const char *, int);
 int		 host_if(const char *, struct server *,
 		    int, in_port_t, const char *, int);
-int		 host(const char *, struct server *,
-		    int, in_port_t, const char *, int);
 int		 is_if_in_group(const char *, const char *);
 
 typedef struct {
@@ -166,7 +164,7 @@ typedef struct {
 } YYSTYPE;
 
 
-#line 170 "parse.c"
+#line 168 "parse.c"
 
 # ifndef YY_CAST
 #  ifdef __cplusplus
@@ -724,11 +722,11 @@ static const yytype_int8 yytranslate[] =
 /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_int16 yyrline[] =
 {
-       0,   135,   135,   136,   137,   138,   139,   140,   143,   161,
-     177,   178,   181,   188,   202,   205,   215,   218,   234,   249,
-     249,   267,   277,   287,   297,   307,   316,   326,   336,   343,
-     366,   370,   373,   376,   379,   382,   385,   388,   391,   397,
-     398,   401,   404,   405
+       0,   133,   133,   134,   135,   136,   137,   138,   141,   159,
+     175,   176,   179,   186,   200,   203,   213,   216,   232,   247,
+     247,   265,   275,   285,   295,   305,   314,   324,   334,   341,
+     364,   368,   371,   374,   377,   380,   383,   386,   389,   395,
+     396,   399,   402,   403
 };
 #endif
 
@@ -1342,13 +1340,13 @@ yyreduce:
   switch (yyn)
     {
   case 7: /* grammar: grammar error '\n'  */
-#line 140 "parse.y"
+#line 138 "parse.y"
                                                 { file->errors++; }
-#line 1348 "parse.c"
+#line 1346 "parse.c"
     break;
 
   case 8: /* varset: STRING '=' STRING  */
-#line 143 "parse.y"
+#line 141 "parse.y"
                                         {
 			char *s = (yyvsp[-2].v.string);
 			while (*s++) {
@@ -1365,11 +1363,11 @@ yyreduce:
 			free((yyvsp[-2].v.string));
 			free((yyvsp[0].v.string));
 		}
-#line 1369 "parse.c"
+#line 1367 "parse.c"
     break;
 
   case 9: /* boolean: STRING  */
-#line 161 "parse.y"
+#line 159 "parse.y"
                          {
 			if (strcasecmp((yyvsp[0].v.string), "1") == 0 ||
 			    strcasecmp((yyvsp[0].v.string), "yes") == 0 ||
@@ -1386,23 +1384,23 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1390 "parse.c"
+#line 1388 "parse.c"
     break;
 
   case 10: /* boolean: ON  */
-#line 177 "parse.y"
+#line 175 "parse.y"
                      { (yyval.v.number) = 1; }
-#line 1396 "parse.c"
+#line 1394 "parse.c"
     break;
 
   case 11: /* boolean: NUMBER  */
-#line 178 "parse.y"
+#line 176 "parse.y"
                          { (yyval.v.number) = (yyvsp[0].v.number); }
-#line 1402 "parse.c"
+#line 1400 "parse.c"
     break;
 
   case 12: /* fcgiport: PORT NUMBER  */
-#line 181 "parse.y"
+#line 179 "parse.y"
                               {
 			if ((yyvsp[0].v.number) <= 0 || (yyvsp[0].v.number) > (int)USHRT_MAX) {
 				yyerror("invalid port: %lld", (yyvsp[0].v.number));
@@ -1410,11 +1408,11 @@ yyreduce:
 			}
 			(yyval.v.port) = (yyvsp[0].v.number);
 		}
-#line 1414 "parse.c"
+#line 1412 "parse.c"
     break;
 
   case 13: /* fcgiport: PORT STRING  */
-#line 188 "parse.y"
+#line 186 "parse.y"
                               {
 			int	 val;
 
@@ -1427,19 +1425,19 @@ yyreduce:
 
 			(yyval.v.port) = val;
 		}
-#line 1431 "parse.c"
+#line 1429 "parse.c"
     break;
 
   case 14: /* main: PREFORK NUMBER  */
-#line 202 "parse.y"
+#line 200 "parse.y"
                                  {
 			gotwebd->prefork_gotwebd = (yyvsp[0].v.number);
 		}
-#line 1439 "parse.c"
+#line 1437 "parse.c"
     break;
 
   case 15: /* main: CHROOT STRING  */
-#line 205 "parse.y"
+#line 203 "parse.y"
                                 {
 			n = strlcpy(gotwebd->httpd_chroot, (yyvsp[0].v.string),
 			    sizeof(gotwebd->httpd_chroot));
@@ -1450,19 +1448,19 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1454 "parse.c"
+#line 1452 "parse.c"
     break;
 
   case 16: /* main: UNIX_SOCKET boolean  */
-#line 215 "parse.y"
+#line 213 "parse.y"
                                       {
 			gotwebd->unix_socket = (yyvsp[0].v.number);
 		}
-#line 1462 "parse.c"
+#line 1460 "parse.c"
     break;
 
   case 17: /* main: UNIX_SOCKET_NAME STRING  */
-#line 218 "parse.y"
+#line 216 "parse.y"
                                           {
 			n = snprintf(gotwebd->unix_socket_name,
 			    sizeof(gotwebd->unix_socket_name), "%s%s",
@@ -1477,11 +1475,11 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1481 "parse.c"
+#line 1479 "parse.c"
     break;
 
   case 18: /* server: SERVER STRING  */
-#line 234 "parse.y"
+#line 232 "parse.y"
                                 {
 			struct server *srv;
 
@@ -1497,11 +1495,11 @@ yyreduce:
 			log_debug("adding server %s", (yyvsp[0].v.string));
 			free((yyvsp[0].v.string));
 		}
-#line 1501 "parse.c"
+#line 1499 "parse.c"
     break;
 
   case 19: /* $@1: %empty  */
-#line 249 "parse.y"
+#line 247 "parse.y"
                                 {
 			struct server *srv;
 
@@ -1517,18 +1515,18 @@ yyreduce:
 			log_debug("adding server %s", (yyvsp[0].v.string));
 			free((yyvsp[0].v.string));
 		}
-#line 1521 "parse.c"
+#line 1519 "parse.c"
     break;
 
   case 20: /* server: SERVER STRING $@1 '{' optnl serveropts2 '}'  */
-#line 263 "parse.y"
+#line 261 "parse.y"
                                             {
 		}
-#line 1528 "parse.c"
+#line 1526 "parse.c"
     break;
 
   case 21: /* serveropts1: REPOS_PATH STRING  */
-#line 267 "parse.y"
+#line 265 "parse.y"
                                     {
 			n = strlcpy(new_srv->repos_path, (yyvsp[0].v.string),
 			    sizeof(new_srv->repos_path));
@@ -1539,11 +1537,11 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1543 "parse.c"
+#line 1541 "parse.c"
     break;
 
   case 22: /* serveropts1: SITE_NAME STRING  */
-#line 277 "parse.y"
+#line 275 "parse.y"
                                    {
 			n = strlcpy(new_srv->site_name, (yyvsp[0].v.string),
 			    sizeof(new_srv->site_name));
@@ -1554,11 +1552,11 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1558 "parse.c"
+#line 1556 "parse.c"
     break;
 
   case 23: /* serveropts1: SITE_OWNER STRING  */
-#line 287 "parse.y"
+#line 285 "parse.y"
                                     {
 			n = strlcpy(new_srv->site_owner, (yyvsp[0].v.string),
 			    sizeof(new_srv->site_owner));
@@ -1569,11 +1567,11 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1573 "parse.c"
+#line 1571 "parse.c"
     break;
 
   case 24: /* serveropts1: SITE_LINK STRING  */
-#line 297 "parse.y"
+#line 295 "parse.y"
                                    {
 			n = strlcpy(new_srv->site_link, (yyvsp[0].v.string),
 			    sizeof(new_srv->site_link));
@@ -1584,11 +1582,11 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1588 "parse.c"
+#line 1586 "parse.c"
     break;
 
   case 25: /* serveropts1: LOGO STRING  */
-#line 307 "parse.y"
+#line 305 "parse.y"
                               {
 			n = strlcpy(new_srv->logo, (yyvsp[0].v.string), sizeof(new_srv->logo));
 			if (n >= sizeof(new_srv->logo)) {
@@ -1598,11 +1596,11 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1602 "parse.c"
+#line 1600 "parse.c"
     break;
 
   case 26: /* serveropts1: LOGO_URL STRING  */
-#line 316 "parse.y"
+#line 314 "parse.y"
                                   {
 			n = strlcpy(new_srv->logo_url, (yyvsp[0].v.string),
 			    sizeof(new_srv->logo_url));
@@ -1613,11 +1611,11 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1617 "parse.c"
+#line 1615 "parse.c"
     break;
 
   case 27: /* serveropts1: CUSTOM_CSS STRING  */
-#line 326 "parse.y"
+#line 324 "parse.y"
                                     {
 			n = strlcpy(new_srv->custom_css, (yyvsp[0].v.string),
 			    sizeof(new_srv->custom_css));
@@ -1628,11 +1626,11 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1632 "parse.c"
+#line 1630 "parse.c"
     break;
 
   case 28: /* serveropts1: LISTEN ON STRING fcgiport  */
-#line 336 "parse.y"
+#line 334 "parse.y"
                                             {
 			if (get_addrs((yyvsp[-1].v.string), new_srv, (yyvsp[0].v.port)) == -1) {
 				yyerror("could not get addrs");
@@ -1640,11 +1638,11 @@ yyreduce:
 			}
 			new_srv->fcgi_socket = 1;
 		}
-#line 1644 "parse.c"
+#line 1642 "parse.c"
     break;
 
   case 29: /* serveropts1: LISTEN ON SOCKET STRING  */
-#line 343 "parse.y"
+#line 341 "parse.y"
                                           {
 			if (!strcasecmp((yyvsp[0].v.string), "off") ||
 			    !strcasecmp((yyvsp[0].v.string), "no")) {
@@ -1668,85 +1666,85 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1672 "parse.c"
+#line 1670 "parse.c"
     break;
 
   case 30: /* serveropts1: MAX_REPOS NUMBER  */
-#line 366 "parse.y"
+#line 364 "parse.y"
                                    {
 			if ((yyvsp[0].v.number) > 0)
 				new_srv->max_repos = (yyvsp[0].v.number);
 		}
-#line 1681 "parse.c"
+#line 1679 "parse.c"
     break;
 
   case 31: /* serveropts1: SHOW_SITE_OWNER boolean  */
-#line 370 "parse.y"
+#line 368 "parse.y"
                                           {
 			new_srv->show_site_owner = (yyvsp[0].v.number);
 		}
-#line 1689 "parse.c"
+#line 1687 "parse.c"
     break;
 
   case 32: /* serveropts1: SHOW_REPO_OWNER boolean  */
-#line 373 "parse.y"
+#line 371 "parse.y"
                                           {
 			new_srv->show_repo_owner = (yyvsp[0].v.number);
 		}
-#line 1697 "parse.c"
+#line 1695 "parse.c"
     break;
 
   case 33: /* serveropts1: SHOW_REPO_AGE boolean  */
-#line 376 "parse.y"
+#line 374 "parse.y"
                                         {
 			new_srv->show_repo_age = (yyvsp[0].v.number);
 		}
-#line 1705 "parse.c"
+#line 1703 "parse.c"
     break;
 
   case 34: /* serveropts1: SHOW_REPO_DESCRIPTION boolean  */
-#line 379 "parse.y"
+#line 377 "parse.y"
                                                 {
 			new_srv->show_repo_description = (yyvsp[0].v.number);
 		}
-#line 1713 "parse.c"
+#line 1711 "parse.c"
     break;
 
   case 35: /* serveropts1: SHOW_REPO_CLONEURL boolean  */
-#line 382 "parse.y"
+#line 380 "parse.y"
                                              {
 			new_srv->show_repo_cloneurl = (yyvsp[0].v.number);
 		}
-#line 1721 "parse.c"
+#line 1719 "parse.c"
     break;
 
   case 36: /* serveropts1: RESPECT_EXPORTOK boolean  */
-#line 385 "parse.y"
+#line 383 "parse.y"
                                            {
 			new_srv->respect_exportok = (yyvsp[0].v.number);
 		}
-#line 1729 "parse.c"
+#line 1727 "parse.c"
     break;
 
   case 37: /* serveropts1: MAX_REPOS_DISPLAY NUMBER  */
-#line 388 "parse.y"
+#line 386 "parse.y"
                                            {
 				new_srv->max_repos_display = (yyvsp[0].v.number);
 		}
-#line 1737 "parse.c"
+#line 1735 "parse.c"
     break;
 
   case 38: /* serveropts1: MAX_COMMITS_DISPLAY NUMBER  */
-#line 391 "parse.y"
+#line 389 "parse.y"
                                              {
 			if ((yyvsp[0].v.number) > 0)
 				new_srv->max_commits_display = (yyvsp[0].v.number);
 		}
-#line 1746 "parse.c"
+#line 1744 "parse.c"
     break;
 
 
-#line 1750 "parse.c"
+#line 1748 "parse.c"
 
       default: break;
     }
@@ -1939,7 +1937,7 @@ yyreturnlab:
   return yyresult;
 }
 
-#line 408 "parse.y"
+#line 406 "parse.y"
 
 
 struct keywords {
@@ -2535,59 +2533,8 @@ getservice(const char *n)
 	return (unsigned short)llval;
 }
 
-struct address *
-host_v4(const char *s)
-{
-	struct in_addr ina;
-	struct sockaddr_in *sain;
-	struct address *h;
-
-	memset(&ina, 0, sizeof(ina));
-	if (inet_pton(AF_INET, s, &ina) != 1)
-		return (NULL);
-
-	if ((h = calloc(1, sizeof(*h))) == NULL)
-		fatal(__func__);
-	sain = (struct sockaddr_in *)&h->ss;
-	got_sockaddr_inet_init(sain, &ina);
-	if (sain->sin_addr.s_addr == INADDR_ANY)
-		h->prefixlen = 0; /* 0.0.0.0 address */
-	else
-		h->prefixlen = -1; /* host address */
-	return (h);
-}
-
-struct address *
-host_v6(const char *s)
-{
-	struct addrinfo hints, *res;
-	struct sockaddr_in6 *sa_in6, *ra;
-	struct address *h = NULL;
-
-	memset(&hints, 0, sizeof(hints));
-	hints.ai_family = AF_INET6;
-	hints.ai_socktype = SOCK_DGRAM; /* dummy */
-	hints.ai_flags = AI_NUMERICHOST;
-	if (getaddrinfo(s, "0", &hints, &res) == 0) {
-		if ((h = calloc(1, sizeof(*h))) == NULL)
-			fatal(__func__);
-		sa_in6 = (struct sockaddr_in6 *)&h->ss;
-		ra = (struct sockaddr_in6 *)res->ai_addr;
-		got_sockaddr_inet6_init(sa_in6, &ra->sin6_addr,
-		    ra->sin6_scope_id);
-		if (memcmp(&sa_in6->sin6_addr, &in6addr_any,
-		    sizeof(sa_in6->sin6_addr)) == 0)
-			h->prefixlen = 0; /* any address */
-		else
-			h->prefixlen = -1; /* host address */
-		freeaddrinfo(res);
-	}
-
-	return (h);
-}
-
 int
-host_dns(const char *s, struct server *new_srv, int max,
+host(const char *s, struct server *new_srv, int max,
     in_port_t port, const char *ifname, int ipproto)
 {
 	struct addrinfo hints, *res0, *res;
@@ -2600,8 +2547,8 @@ host_dns(const char *s, struct server *new_srv, int max,
 		return (cnt);
 
 	memset(&hints, 0, sizeof(hints));
-	hints.ai_family = PF_UNSPEC;
-	hints.ai_socktype = SOCK_DGRAM; /* DUMMY */
+	hints.ai_family = AF_UNSPEC;
+	hints.ai_socktype = SOCK_STREAM; /* DUMMY */
 	hints.ai_flags = AI_ADDRCONFIG;
 	error = getaddrinfo(s, NULL, &hints, &res0);
 	if (error == EAI_AGAIN || error == EAI_NODATA || error == EAI_NONAME)
@@ -2634,7 +2581,6 @@ host_dns(const char *s, struct server *new_srv, int max,
 		if (ipproto != -1)
 			h->ipproto = ipproto;
 		h->ss.ss_family = res->ai_family;
-		h->prefixlen = -1; /* host address */
 
 		if (res->ai_family == AF_INET) {
 			struct sockaddr_in *ra;
@@ -2701,7 +2647,6 @@ host_if(const char *s, struct server *new_srv, int max,
 		if (ipproto != -1)
 			h->ipproto = ipproto;
 		h->ss.ss_family = af;
-		h->prefixlen = -1; /* host address */
 
 		if (af == AF_INET) {
 			struct sockaddr_in *ra;
@@ -2734,41 +2679,6 @@ host_if(const char *s, struct server *new_srv, int max,
 	return (cnt);
 }
 
-int
-host(const char *s, struct server *new_srv, int max,
-    in_port_t port, const char *ifname, int ipproto)
-{
-	struct address *h;
-
-	h = host_v4(s);
-
-	/* IPv6 address? */
-	if (h == NULL)
-		h = host_v6(s);
-
-	if (h != NULL) {
-		if (port)
-			h->port = port;
-		if (ifname != NULL) {
-			if (strlcpy(h->ifname, ifname, sizeof(h->ifname)) >=
-			    sizeof(h->ifname)) {
-				log_warnx("%s: interface name truncated",
-				    __func__);
-				free(h);
-				return (-1);
-			}
-		}
-		if (ipproto != -1)
-			h->ipproto = ipproto;
-
-		if (add_addr(new_srv, h))
-			return -1;
-		return (1);
-	}
-
-	return (host_dns(s, new_srv, max, port, ifname, ipproto));
-}
-
 int
 is_if_in_group(const char *ifname, const char *groupname)
 {
