@@ -125,7 +125,7 @@ got_fileindex_entry_update(struct got_fileindex_entry *ie,
 	}
 
 	if (blob_sha1) {
-		memcpy(ie->blob_sha1, blob_sha1, SHA1_DIGEST_LENGTH);
+		memmove(ie->blob_sha1, blob_sha1, SHA1_DIGEST_LENGTH);
 		ie->flags &= ~GOT_FILEIDX_F_NO_BLOB;
 	} else
 		ie->flags |= GOT_FILEIDX_F_NO_BLOB;
@@ -749,8 +749,10 @@ got_fileindex_read(struct got_fileindex *fileindex, FILE *infile)
 		if (err)
 			return err;
 		err = add_entry(fileindex, ie);
-		if (err)
+		if (err) {
+			got_fileindex_entry_free(ie);
 			return err;
+		}
 	}
 
 	n = fread(sha1_expected, 1, sizeof(sha1_expected), infile);
